/***************************************************************************
 *            BackupFilesPanel.cc
 *
 *  Tue Mar  1 00:24:16 2005
 *  Copyright  2005  Chris Wilson
 *  anjuta@qwirx.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <wx/dir.h>
#include <wx/filename.h>

#include "BackupFilesPanel.h"

LocalFileTreeNode::LocalFileTreeNode() {
	checked = FALSE;
}

LocalFileTreeNode::~LocalFileTreeNode() {
	
}

void LocalFileTreeNode::AddChildren() {
	ShowChildren(NULL);
}

void LocalFileTreeNode::ShowChildren(wxListCtrl *targetList) {
	theRootNode->theTree->SetCursor(*wxHOURGLASS_CURSOR);
	wxSafeYield();
	
	try {
		ScanChildrenAndList(targetList, false);
	} catch (std::exception& e) {
		theRootNode->theTree->SetCursor(*wxSTANDARD_CURSOR);
		throw(e);
	}
	
	theRootNode->theTree->SetCursor(*wxSTANDARD_CURSOR);
}

void LocalFileTreeNode::ScanChildrenAndList(wxListCtrl *targetList,
	bool recursive) 
{
	// Build our full path
	wxArrayString theParentDirList;
	
	for (LocalFileTreeNode *theNodePointer = this; theNodePointer != NULL; 
		theNodePointer = theNodePointer->theParentNode) {
		theParentDirList.Insert(theNodePointer->fileName, 0);
	}
	
	wxString theFullPath;
	
	for (size_t i = 0; i < theParentDirList.GetCount(); i++) {
		theFullPath.Append("/");
		theFullPath.Append(theParentDirList.Item(i));
	}
	
	wxDir dir(theFullPath);
	
	if ( !dir.IsOpened() )
    {
        // deal with the error here - wxDir would already log an error message
        // explaining the exact reason of the failure
        return;
    }

	// delete any existing children of the parent
	theTree->DeleteChildren(treeId);
	
	if (targetList != NULL)
		targetList->DeleteAllItems();
		
	wxString theCurrentFileName;
	
	bool cont = dir.GetFirst(&theCurrentFileName);
    while ( cont )
    {
		// add to the tree
		LocalFileTreeNode *newNode = new LocalFileTreeNode();
		newNode->treeId = theTree->AppendItem(this->treeId,
			theCurrentFileName, -1, -1, newNode);
		newNode->theTree = theTree;
		newNode->theRootNode = theRootNode;
		newNode->theParentNode = this;
		newNode->fileName = theCurrentFileName;
		
		wxFileName fileNameObject = wxFileName(theFullPath);
		fileNameObject.AppendDir(theCurrentFileName);
		newNode->isDirectory = fileNameObject.DirExists();

		if (newNode->isDirectory)
		{
			if (recursive) {
				newNode->ScanChildrenAndList(NULL, false);
			} else {
				theTree->SetItemHasChildren(newNode->treeId, TRUE);
			}
		}

        cont = dir.GetNext(&theCurrentFileName);
    }
}

BEGIN_EVENT_TABLE(BackupFilesPanel, wxPanel)
	EVT_TREE_ITEM_EXPANDING(ID_Backup_Files_Tree, 
		BackupFilesPanel::OnTreeNodeExpand)
	EVT_TREE_ITEM_COLLAPSING(ID_Backup_Files_Tree, 
		BackupFilesPanel::OnTreeNodeCollapse)
	EVT_TREE_ITEM_ACTIVATED(ID_Backup_Files_Tree, 
		BackupFilesPanel::OnTreeNodeActivated)
END_EVENT_TABLE()

BackupFilesPanel::BackupFilesPanel(ClientConfig *config,
	wxWindow* parent, wxWindowID id,
	const wxPoint& pos, const wxSize& size,
	long style, const wxString& name)
	: wxPanel(parent, id, pos, size, style, name)
{
	theConfig = config;
	
	wxBoxSizer *topSizer = new wxBoxSizer( wxVERTICAL );
	
	theBackupFileTree = new wxTreeCtrl(this, ID_Backup_Files_Tree,
		wxDefaultPosition, wxDefaultSize, wxTR_HAS_BUTTONS);

	topSizer->Add(theBackupFileTree, 1, wxGROW, 8);
	
	SetSizer( topSizer );
	topSizer->SetSizeHints( this );

	LocalFileTreeNode *rootNode = new LocalFileTreeNode();
	rootNode->fileName			= "";
	theTreeRootItem				= theBackupFileTree->AddRoot(
		wxString("/ (local root)"), -1, -1, rootNode);
	rootNode->treeId			= theTreeRootItem;
	rootNode->theTree			= theBackupFileTree;
	rootNode->theConfig			= config;
	rootNode->theRootNode		= rootNode;
	rootNode->theParentNode		= NULL;
	theBackupFileTree->SetItemHasChildren(theTreeRootItem, TRUE);

	wxMemoryDC theDC;

	{
		theUncheckedImage = wxBitmap(16, 16, -1);
		theDC.SelectObject(theUncheckedImage);
		theDC.Clear();
		theDC.DrawRectangle(2, 2, 12, 12);
		theTreeImages.Add(theUncheckedImage.ConvertToImage());
	}
	
	{
		wxBitmap theCheckedImage = wxBitmap(16, 16, -1);
		theDC.SelectObject(theCheckedImage);
		theDC.Clear();
		theDC.DrawRectangle(2, 2, 12, 12);
		for (int offset = -1; offset < 2; offset++) {
			theDC.DrawLine(5, 7 + offset, 7, 9 + offset);
			theDC.DrawLine(7, 9 + offset, 11, 5 + offset);
		}
		theTreeImages.Add(theCheckedImage  .ConvertToImage());
	}
	
	theBackupFileTree->SetImageList(&theTreeImages);
	theBackupFileTree->SetItemImage(theTreeRootItem, 1, 
		wxTreeItemIcon_Normal);
}

void BackupFilesPanel::OnTreeNodeExpand(wxTreeEvent& event)
{
	wxTreeItemId item = event.GetItem();
	LocalFileTreeNode *node = 
		(LocalFileTreeNode *)(theBackupFileTree->GetItemData(item));
	node->AddChildren();
}

void BackupFilesPanel::OnTreeNodeCollapse(wxTreeEvent& event)
{

}

void BackupFilesPanel::OnTreeNodeActivated(wxTreeEvent& event)
{
	wxTreeItemId item = event.GetItem();
	
	LocalFileTreeNode *node = 
		(LocalFileTreeNode *)(theBackupFileTree->GetItemData(item));
	
	node->SetChecked(! node->IsChecked() );
	
	theBackupFileTree->SetItemImage(item, 
		node->IsChecked() ? 1 : 0, wxTreeItemIcon_Normal);
}
