/***************************************************************************
 *            ClientInfoPanel.cc
 *
 *  Tue Mar  1 00:16:12 2005
 *  Copyright  2005  Chris Wilson
 *  anjuta@qwirx.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <wx/wx.h>
#include <wx/notebook.h>

#include "ClientInfoPanel.h"

class ParamPanel : public wxPanel {
	public:
	ParamPanel(
		wxWindow* parent, wxWindowID id = -1,
		const wxPoint& pos = wxDefaultPosition, 
		const wxSize& size = wxDefaultSize,
		long style = wxTAB_TRAVERSAL, 
		const wxString& name = "panel");
	
	wxTextCtrl* AddParam(const char * pLabel, StringProperty* pProp, int ID);
	wxTextCtrl* AddParam(const char * pLabel, IntProperty*    pProp, 
		const char *format, int ID);
	wxCheckBox* AddParam(const char * pLabel, BoolProperty*   pProp, int ID);
	
	private:
	wxGridSizer *mpSizer;
};

ParamPanel::ParamPanel(
	wxWindow* parent, wxWindowID id,
	const wxPoint& pos, const wxSize& size,
	long style, const wxString& name)
	: wxPanel(parent, id, pos, size, style, name)
{
	wxBoxSizer *pBasicBox = new wxBoxSizer(wxVERTICAL);
	SetSizer(pBasicBox, true);
	
	mpSizer = new wxGridSizer(2, 4, 4);
	pBasicBox->Add(mpSizer, 0, wxGROW | wxALL, 8);
}

wxTextCtrl* ParamPanel::AddParam(const char * pLabel, StringProperty* pProp, 
	int ID)
{
	wxString ValueString;
	if (const std::string * value = pProp->Get())
		ValueString = value->c_str();
	
	wxTextCtrl *pCtrl = new wxTextCtrl(this, ID, ValueString);
	::AddParam(this, pLabel, pCtrl, true, mpSizer);
	
	return pCtrl;
}

wxTextCtrl* ParamPanel::AddParam(const char * pLabel, IntProperty* pProp, 
	const char * pFormat, int ID)
{
	wxString ValueString = "";
	if (const int * Value = pProp->Get()) {
		ValueString.Printf(pFormat, *Value);
	}
	
	wxTextCtrl *pCtrl = new wxTextCtrl(this, ID, ValueString);
	::AddParam(this, pLabel, pCtrl, true, mpSizer);
	
	return pCtrl;
}

wxCheckBox* ParamPanel::AddParam(const char * pLabel, BoolProperty* pProp, 
	int ID)
{
	wxCheckBox *pCtrl = new wxCheckBox(this, ID, "Enabled");
	::AddParam(this, pLabel, pCtrl, true, mpSizer);

	if (const bool * value = pProp->Get()) 
		pCtrl->SetValue(*value);
	
	return pCtrl;
}

BEGIN_EVENT_TABLE(ClientInfoPanel, wxPanel)
	EVT_TEXT(ID_ClientProp_StoreHostname,   ClientInfoPanel::OnPropertyChange)
	EVT_TEXT(ID_ClientProp_StorePort,       ClientInfoPanel::OnPropertyChange)
	EVT_TEXT(ID_ClientProp_AccountNumber,   ClientInfoPanel::OnPropertyChange)
	EVT_TEXT(ID_ClientProp_KeysFile,        ClientInfoPanel::OnPropertyChange)
	EVT_TEXT(ID_ClientProp_CertificateFile, ClientInfoPanel::OnPropertyChange)
	EVT_TEXT(ID_ClientProp_PrivateKeyFile,  ClientInfoPanel::OnPropertyChange)
	EVT_TEXT(ID_ClientProp_TrustedCAsFile,  ClientInfoPanel::OnPropertyChange)
	EVT_TEXT(ID_ClientProp_DataDirectory,   ClientInfoPanel::OnPropertyChange)
	EVT_TEXT(ID_ClientProp_NotifyScript,    ClientInfoPanel::OnPropertyChange)
	EVT_TEXT(ID_ClientProp_UpdateStoreInterval, ClientInfoPanel::OnPropertyChange)
	EVT_TEXT(ID_ClientProp_MinimumFileAge,  ClientInfoPanel::OnPropertyChange)
	EVT_TEXT(ID_ClientProp_MaxUploadWait,   ClientInfoPanel::OnPropertyChange)
	EVT_TEXT(ID_ClientProp_FileTrackingSizeThreshold, ClientInfoPanel::OnPropertyChange)
	EVT_TEXT(ID_ClientProp_DiffingUploadSizeThreshold, ClientInfoPanel::OnPropertyChange)
	EVT_TEXT(ID_ClientProp_MaximumDiffingTime, ClientInfoPanel::OnPropertyChange)
	EVT_CHECKBOX(ID_ClientProp_ExtendedLogging, ClientInfoPanel::OnPropertyChange)
	EVT_TEXT(ID_ClientProp_SyncAllowScript, ClientInfoPanel::OnPropertyChange)
	EVT_TEXT(ID_ClientProp_CommandSocket,   ClientInfoPanel::OnPropertyChange)
	EVT_TEXT(ID_ClientProp_PidFile,         ClientInfoPanel::OnPropertyChange)
END_EVENT_TABLE()

ClientInfoPanel::ClientInfoPanel(ClientConfig *pConfig,
	wxWindow* parent, wxWindowID id,
	const wxPoint& pos, const wxSize& size,
	long style, const wxString& name)
	: wxPanel(parent, id, pos, size, style, name)
{
	mpConfig = pConfig;
	
	wxBoxSizer *topSizer = new wxBoxSizer( wxVERTICAL );
	
	wxNotebook *pClientPropsNotebook = new wxNotebook(this, -1);
	topSizer->Add(pClientPropsNotebook, 1, wxGROW | wxALL, 8);
	
	ParamPanel *pBasicPanel = new ParamPanel(pClientPropsNotebook);
	pClientPropsNotebook->AddPage(pBasicPanel, "Basic");
	
	ParamPanel *pAdvancedPanel = new ParamPanel(pClientPropsNotebook);
	pClientPropsNotebook->AddPage(pAdvancedPanel, "Advanced");

	mpStoreHostnameCtrl = pBasicPanel->AddParam("Store Host:", 
		pConfig->StoreHostname, ID_ClientProp_StoreHostname);

	mpStorePortCtrl = pBasicPanel->AddParam("Store Port:", 
		pConfig->StorePort, "%d", ID_ClientProp_StorePort);
	
	mpAccountNumberCtrl = pBasicPanel->AddParam("Account Number:", 
		pConfig->AccountNumber, "0x%x", ID_ClientProp_AccountNumber);
		
	mpKeysFileCtrl = pBasicPanel->AddParam("Keys File:", 
		pConfig->KeysFile, ID_ClientProp_KeysFile);
		
	mpCertificateFileCtrl = pBasicPanel->AddParam("Certificate File:", 
		pConfig->CertificateFile, ID_ClientProp_CertificateFile);

	mpPrivateKeyFileCtrl = pBasicPanel->AddParam("Private Key File:", 
		pConfig->PrivateKeyFile, ID_ClientProp_PrivateKeyFile);

	mpTrustedCAsFileCtrl = pBasicPanel->AddParam("Trusted CAs File:", 
		pConfig->TrustedCAsFile, ID_ClientProp_TrustedCAsFile);

	mpDataDirectoryCtrl = pBasicPanel->AddParam("Data Directory:", 
		pConfig->DataDirectory, ID_ClientProp_DataDirectory);

	mpNotifyScriptCtrl = pBasicPanel->AddParam("Notify Script:", 
		pConfig->NotifyScript, ID_ClientProp_NotifyScript);
	
	mpUpdateStoreIntervalCtrl = pAdvancedPanel->AddParam(
		"Update Store Interval:", pConfig->UpdateStoreInterval, 
		"%d", ID_ClientProp_UpdateStoreInterval);
		
	mpMinimumFileAgeCtrl = pAdvancedPanel->AddParam("Minimum File Age:", 
		pConfig->MinimumFileAge, "%d", ID_ClientProp_MinimumFileAge);

	mpMaxUploadWaitCtrl = pAdvancedPanel->AddParam("Maximum Upload Wait:", 
		pConfig->MaxUploadWait, "%d", ID_ClientProp_MaxUploadWait);

	mpFileTrackingSizeThresholdCtrl = pAdvancedPanel->AddParam(
		"File Tracking Size Threshold:", pConfig->FileTrackingSizeThreshold, 
		"%d", ID_ClientProp_FileTrackingSizeThreshold);
		
	mpDiffingUploadSizeThresholdCtrl = pAdvancedPanel->AddParam(
		"Diffing Upload Size Threshold:", pConfig->DiffingUploadSizeThreshold, 
		"%d", ID_ClientProp_DiffingUploadSizeThreshold);

	mpMaxDiffingTimeCtrl = pAdvancedPanel->AddParam("Maximum Diffing Time:", 
		pConfig->MaximumDiffingTime, "%d", ID_ClientProp_MaximumDiffingTime);

	mpExtendedLoggingCtrl = pBasicPanel->AddParam("Extended Logging:",
		pConfig->ExtendedLogging, ID_ClientProp_ExtendedLogging);

	mpSyncAllowScriptCtrl = pAdvancedPanel->AddParam("Sync Allow Script:", 
		pConfig->SyncAllowScript, ID_ClientProp_SyncAllowScript);

	mpCommandSocketCtrl = pAdvancedPanel->AddParam("Command Socket:", 
		pConfig->CommandSocket, ID_ClientProp_CommandSocket);

	mpPidFileCtrl = pAdvancedPanel->AddParam("Process ID File:", 
		pConfig->PidFile, ID_ClientProp_PidFile);
		
	SetSizer( topSizer );
	topSizer->SetSizeHints( this );
}

bool ClientInfoPanel::UpdateStringProperty(wxTextCtrl* pCtrl, 
	StringProperty* pProp) 
{
	wxString tempString = pCtrl->GetValue();
	if (tempString.Length() == 0) {
		pProp->Clear();
	} else {
		pProp->Set(tempString.c_str());
	}
	return TRUE;
}

bool ClientInfoPanel::UpdateNumericProperty(wxTextCtrl* pCtrl, 
	IntProperty* pProp) 
{
	wxString tempString = pCtrl->GetValue();
	if (tempString.Length() == 0) {
		pProp->Clear();
		return TRUE;
	}
	
	unsigned int tempValue;
	char *endptr;
	
	if (tempString.compare(0, 2, "0x") == 0) {
		tempValue = strtol(tempString.c_str() + 2, &endptr, 16);
	} else {
		tempValue = strtol(tempString.c_str(), &endptr, 10);
	}
	
	if (*endptr != '\0') {
		return FALSE;	
	}

	pProp->Set(tempValue);
	return TRUE;
}

bool ClientInfoPanel::UpdateBooleanProperty(wxCheckBox* pCtrl, 
	BoolProperty* pProp) 
{
	pProp->Set(pCtrl->IsChecked());
	return TRUE;
}


void ClientInfoPanel::OnPropertyChange(wxCommandEvent &event)
{
	// only one of these is validly typed, depending on the actual source 
	// of the event, determined by ID below.
	wxTextCtrl *pText  = (wxTextCtrl*)( event.GetEventObject() );
	wxCheckBox *pCheck = (wxCheckBox*)( event.GetEventObject() );
	
	switch(event.GetId()) {
	case ID_ClientProp_StoreHostname:
		UpdateStringProperty(pText, mpConfig->StoreHostname);
		break;
	
	case ID_ClientProp_StorePort:
		UpdateNumericProperty(pText, mpConfig->StorePort);
		break;

	case ID_ClientProp_AccountNumber:
		UpdateNumericProperty(pText, mpConfig->AccountNumber);
		break;

	case ID_ClientProp_KeysFile:
		UpdateStringProperty(pText, mpConfig->KeysFile);
		break;

	case ID_ClientProp_CertificateFile:
		UpdateStringProperty(pText, mpConfig->CertificateFile);
		break;

	case ID_ClientProp_PrivateKeyFile:
		UpdateStringProperty(pText, mpConfig->PrivateKeyFile);
		break;
	
	case ID_ClientProp_TrustedCAsFile:
		UpdateStringProperty(pText, mpConfig->TrustedCAsFile);
		break;

	case ID_ClientProp_DataDirectory:
		UpdateStringProperty(pText, mpConfig->DataDirectory);
		break;

	case ID_ClientProp_NotifyScript:
		UpdateStringProperty(pText, mpConfig->NotifyScript);
		break;

	case ID_ClientProp_UpdateStoreInterval:
		UpdateNumericProperty(pText, mpConfig->UpdateStoreInterval);
		break;
	
	case ID_ClientProp_MinimumFileAge:
		UpdateNumericProperty(pText, mpConfig->MinimumFileAge);
		break;

	case ID_ClientProp_MaxUploadWait:
		UpdateNumericProperty(pText, mpConfig->MaxUploadWait);
		break;

	case ID_ClientProp_FileTrackingSizeThreshold:
		UpdateNumericProperty(pText, mpConfig->FileTrackingSizeThreshold);
		break;

	case ID_ClientProp_DiffingUploadSizeThreshold:
		UpdateNumericProperty(pText, mpConfig->DiffingUploadSizeThreshold);
		break;

	case ID_ClientProp_MaximumDiffingTime:
		UpdateNumericProperty(pText, mpConfig->MaximumDiffingTime);
		break;

	case ID_ClientProp_ExtendedLogging:
		UpdateBooleanProperty(pCheck, mpConfig->ExtendedLogging);
		break;

	case ID_ClientProp_SyncAllowScript:
		UpdateStringProperty(pText, mpConfig->SyncAllowScript);
		break;

	case ID_ClientProp_CommandSocket:
		UpdateStringProperty(pText, mpConfig->CommandSocket);
		break;

	case ID_ClientProp_PidFile:
		UpdateStringProperty(pText, mpConfig->PidFile);
		break;

	default:
		wxMessageBox("Update to unknown control", "Internal error", 
			wxOK | wxICON_ERROR, this);
		
	}
}
